/* xfpovray callback functions 
** 
** All callback function names are of the form FunctionNameCB.c
**
*/


#include "forms.h"
#include "xfpovray.h"
#include "povray-defaults.h"
#include "wids_struct.h"
#include "pov_struct.h"
#include <signal.h>
#include <sys/wait.h>

#include <stdlib.h>
 
#define TIME 0.01
#define PUSHED 1
#define RELEASED 0

extern int start_x;
extern int start_y;
extern int have_geometry;  

extern struct widget_ids wids;
extern struct pov_options pov;
 
extern int povray_pid;
extern int auto_view;

extern char POV_INIFILE[1024];
extern char STREAM_FILE[1024];
extern char FATAL_FILE[1024];
extern char PREVIEW_FILE[1024];
 
extern int  loaded_inifile;
extern void write_config();
extern int  povray_pid;
extern int  auto_view;
extern void show_image();
extern int  PovCheckCB();
 
 
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
void AbortCB(FL_OBJECT *ob, long data)
{
    if (povray_pid == 0) 
       return;

    kill(povray_pid, SIGINT);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
void ClearPartialImageCB(FL_OBJECT *ob, long data)
{
    fl_set_input(wids.wid_start_column, "");
    fl_set_input(wids.wid_start_row, "");
    fl_set_input(wids.wid_end_column, "");
    fl_set_input(wids.wid_end_row, "");

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
/* Configuration file options */
void ConfigCB(FL_OBJECT *ob, long data)
{

    int i, submenu;
    const char *save_config_file;
    char temp[256];
    char *t;
    const char *temp2;
         
    submenu = fl_get_menu(ob);

    switch (submenu) {
           case 1: /* Show */
                   /* Config file is shown in the option area, so turn off
                   ** any option radio button that might be selected, then hide
                   ** the current option group
                   */
                   for (i=0; i<4; i++)
                       fl_set_button(wids.opt[i], RELEASED);  
                   fl_hide_object(wids.current_grp);
     
                   wids.current_grp = wids.config_grp;
                   if (loaded_inifile != 1)
                      write_config();  
                   fl_clear_browser(wids.config_browser);
                   fl_load_browser(wids.config_browser, POV_INIFILE);     
    
                   fl_show_object(wids.current_grp);
                   break;
           case 2: /* Save */
                   fl_set_goodies_font(FL_BOLD_STYLE, 12);
                   t = strtok((char *) fl_get_filename(), "."); 
                   if (t == NULL)
                      sprintf(temp, "default.ini");
                   else
                      sprintf(temp, "%s.ini", t);
       
                   save_config_file = fl_show_input(
                       "Enter configuration filename", temp);
    
                   sprintf(temp, "cp %s %s", POV_INIFILE, save_config_file);

                   system(temp);
                   break;
           case 3: /* Load */        
                   temp2 = fl_show_fselector("Enter filename:", 
                       POV_DIR, "*.ini", "");  
                   if (temp2 == NULL) 
                      return;
                   else {
                      loaded_inifile = 1;
                      sprintf(POV_INIFILE, "%s", temp2);         
                   }
                   break;
           default: perror("xfpovray: ConfigCB");
    } /* endswitch */
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
/* For the fancy "About" widget */
void ContinueCB(FL_OBJECT *ob, long data)
{
    int r, val;
    int num_colors = 10;
    int colors[10] = {FL_RED,
                      FL_GREEN,
                      FL_YELLOW,
                      FL_BLUE,
                      FL_MAGENTA,
                      FL_SLATEBLUE,
                      FL_ORCHID,
                      FL_DARKGOLD,
                      FL_TOMATO,
                      FL_WHITE};
    static FL_Coord dx = 1, dy = 1;
    FL_Coord x, y, w, h;  


    
    fl_set_timer(wids.tim, TIME);
    fl_set_object_callback(wids.tim, ContinueCB, 0);

    /* Set a random color for tc */
    srand(time(0));
    r = rand();
    val = (int) ( (r / (float) RAND_MAX) * (num_colors) ); 
       
    fl_set_object_lcol(wids.tc, colors[val]);


    /* Set the text position */
    fl_get_object_geometry(wids.tc, &x, &y, &w, &h);
    
    if ( (x + dx) < 0 || (x + w + dx) >= wids.tc->form->w)
       dx = -dx;
    if ( (y + dy) < 0 || (y + (h+140) + dy) >= wids.tc->form->h)
       dy = -dy;
    
    x += dx;
    y += dy;

    fl_set_object_position(wids.tc, x ,y);


}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
void DismissAboutCB(FL_OBJECT *ob, long data)
{
    fl_hide_form(wids.about_form);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
void DismissMessageCB(FL_OBJECT *ob, long data)
{

    switch (data) {
           case 0: fl_hide_form(wids.text_form);
                   break;
           case 1: fl_hide_form(wids.fatal_form);
                   break;
           default: perror("xfpovray: DismissMessageCB");
    }
    

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
/* Pops up a file selection box */
void GetFileCB(FL_OBJECT *ob, long data)
{   
    const char *temp;
    char command[256];

 
    switch (data) {
       case 0: /* Scene File button */
            temp = pov.input_file_name;
            pov.input_file_name = fl_show_fselector("Enter pov filename:", 
                POV_DIR, SCENE_FILTER, "");  
            if (pov.input_file_name == NULL) {
               /* Restore old file */
               pov.input_file_name = temp; 
               return;
            }   
            else
               fl_set_input(wids.wid_file_input, pov.input_file_name);
            break;
       case 1: /* Edit Scene button */
            temp = fl_show_fselector("Enter filename:", 
                POV_DIR, SCENE_FILTER, "");  
            if (temp == NULL) 
               return;
            else {
               sprintf(command, "%s %s &", POV_EDITOR, temp);
               system(command);
            }
            break;
       case 2: /* View Image button */
            temp = fl_show_fselector("Enter filename:", 
                POV_DIR, IMAGE_FILTER, "");  
            if (temp == NULL) 
               return;
            else {
               sprintf(command, "%s %s &", DISPLAY_COMMAND, temp);
               system(command);            
            }
            break;
        default: perror("xfpovray: GetFileCB");

    } /* endswitch */          

}

 
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
void HelpCB(FL_OBJECT *ob, long data)
{
    int submenu;
    

    submenu = fl_get_menu(ob);

    switch (submenu) {
           case 1: fl_set_goodies_font(FL_BOLD_STYLE, 12);
                   fl_show_message("Select a Scene File",
                       "Set Options for the Scene", 
                       "Render Image!");
                   break;
           case 2: fl_show_form(wids.about_form, 
                       FL_PLACE_CENTERFREE, FL_FULLBORDER, "xfpovray");
                   break;
           default: perror("xfpovray: HelpCB");
    } 
    
 
} 

 
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
/* Write the config file, then invoke POV-ray */
void PovCB(FL_OBJECT *ob, long data)
{
    char command[256];
    int pid;
    FILE *stream_file;
    FILE *fatal_file;
           
    if (pov.input_file_name == NULL) {

       fl_set_goodies_font(FL_BOLD_STYLE, 12);
       if (fl_show_question("You have not entered", "a scene file.",
          "Select scene file now?") == 1) {
        
          pov.input_file_name = fl_show_fselector("Enter pov filename:", 
              POV_DIR, SCENE_FILTER, "");  
          if (pov.input_file_name == NULL) 
             return;
          else
             fl_set_input(wids.wid_file_input, pov.input_file_name);
       }
       else 
          return;
    }       

    if (loaded_inifile != 1) 
       write_config();
    else
       loaded_inifile = 0;

    /* Execute POV-Ray as a child process so we can interrupt a 
    ** render by sending SIGINT. 
    */
    pid = fork();
    
    if (pid == 0) {
       /* In child process; invoke POV-Ray */

       /* Open new files for the current run of POV-Ray.  If these files
       ** have non-zero length when POV-Ray is done, display them
       */ 
       if ((stream_file = fopen(STREAM_FILE, "w")) == NULL) {
          perror("xfpovray: PovCB");
          exit (1);
       }
       fclose(stream_file);
       
       if ((fatal_file = fopen(FATAL_FILE, "w")) == NULL) {
          perror("xfpovray: PovCB");
          exit (1);
       }
       fclose(fatal_file);

       sprintf(command, "%s", POV_COMMAND);
       if (execlp (command, "xfpovray", POV_INIFILE, NULL) == -1) {
        
          printf ("Error executing command\n %s\n", command);
          perror ("xfpovray");
          /* sigchld catcher handles the error */
          _exit (255);
       }
    }
    else if (pid > 0) {
       /* In parent process */
       
       povray_pid = pid; 
                 
    }
    else {
        printf ("Error: PovCB: fork() failed!\n");
        perror ("xfpovray");
        exit (1);
    }

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
int PovCheckCB(XEvent *ev, void *data)
{
    int status, code;
    char temp[256];
    FILE *stream_file;
    FILE *fatal_file;

    
    /* Idle */
    if (povray_pid == 0) {
       return;
    }

    status = waitpid(povray_pid, &code, WNOHANG);

    /* If status == 0, POV-Ray is running; if status < 0, POV-Ray is
    ** done or has been aborted.  We need not be concerned with 'code'.
    */ 
    if (status < 0) {

       povray_pid = 0;

       /* If verbose output is on, display a Message box, which will 
       ** include any error messages, else display an Error box if needed.
       */
       if (pov.redirect_on == 1) {

	  if ((stream_file = fopen(STREAM_FILE, "r")) == NULL) {
             perror("xfpovray: PovCB");
             exit (1);
	  }
	  else {
             if (fgets(temp, sizeof(temp), stream_file) != NULL) {
 		fl_set_form_position(wids.text_form, 0, 0);
		fl_show_form(wids.text_form, FL_PLACE_POSITION, 
		    FL_TRANSIENT, "Messages");
		fl_load_browser(wids.text, STREAM_FILE);
             }
          fclose(stream_file);    
          }

       } /* endif */
       else {
       
	  if ((fatal_file = fopen(FATAL_FILE, "r")) == NULL) {
             perror("xfpovray: PovCB");
             exit (1);
	  }
	  else {
             if (fgets(temp, sizeof(temp), fatal_file) != NULL) {
               fl_set_form_position(wids.fatal_form, 0, 0);
               fl_show_form(wids.fatal_form, FL_PLACE_POSITION, 
        	   FL_TRANSIENT, "Error");
               fl_load_browser(wids.fatal_text, FATAL_FILE);
             }
             fclose(fatal_file);    
	  }

       } /* endelse */

       /* Display image using viewer? */
       if ((auto_view == 1) && (pov.display_during_render != 1))
	  show_image();

    } /* endif */   

 
}
 

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
void QualityCB(FL_OBJECT *ob, long data)
{
    switch (data) {
           case 0: pov.quality = (int) fl_get_slider_value(ob);
                   break;
           case 1: pov.aa_depth = (int) fl_get_slider_value(ob);
                   break;
    }
                   
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
void QualityResetCB(FL_OBJECT *ob, long data)
{
    switch (data) {
           case 0: fl_set_slider_value(wids.quality_slider, POV_QUALITY);
                   pov.quality = POV_QUALITY;
                   break;
           case 1: fl_set_slider_value(wids.aa_slider, POV_AA_DEPTH);
                   pov.aa_depth = POV_AA_DEPTH;
                   break;
    }

}
 

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
void QuitCB(FL_OBJECT *ob, long data)
{
    fl_finish();
    exit (0);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
/* The option radio button callback; sets the current group and shows it */
void RadioCB(FL_OBJECT *ob, long data)
{

    fl_hide_object(wids.current_grp);

    if (fl_get_button(wids.opt[0]) == 1)
       wids.current_grp = wids.parsing_grp;
       
    if (fl_get_button(wids.opt[1]) == 1) 
       wids.current_grp = wids.output_grp;

    if (fl_get_button(wids.opt[2]) == 1) 
       wids.current_grp = wids.tracing_grp;

    if (fl_get_button(wids.opt[3]) == 1)
       wids.current_grp = wids.animation_grp;
                       
    fl_show_object(wids.current_grp);

}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
/* Set image height or width */
void SetImageSizeCB(FL_OBJECT *ob, long data)
{
    char temp[10];

    if (data == 0) {
       fl_set_input(wids.wid_height_input, fl_get_menu_text(ob));

       if (strcmp(fl_get_input(wids.wid_height_input), "") == 0) {
	  sprintf(temp, "%d", POV_HEIGHT);
	  fl_set_input(wids.wid_height_input, temp);
	  pov.height = POV_HEIGHT;
       }
       else
	  pov.height = atoi(fl_get_input(wids.wid_height_input));
    }
    if (data == 1) {
       fl_set_input(wids.wid_width_input, fl_get_menu_text(ob));

       if (strcmp(fl_get_input(wids.wid_width_input), "") == 0) {
	  sprintf(temp, "%d", POV_WIDTH);
	  fl_set_input(wids.wid_width_input, temp);
	  pov.width = POV_WIDTH;
       }
       else
	  pov.width = atoi(fl_get_input(wids.wid_width_input)); 
    }
    
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
void SetMosaicCB(FL_OBJECT *ob, long data)
{

    if (data == 0) {
       fl_set_input(wids.mosaic_start, fl_get_menu_text(ob));
    }
    if (data == 1) {
       fl_set_input(wids.mosaic_end, fl_get_menu_text(ob));
    }
   
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
void TogglePixmapCB(FL_OBJECT *ob, long data)
{
    int i;

    if ( wids.current_grp != wids.pixmap_grp) {

       /* Release any option buttons that might be selected */
       for (i=0; i<4; i++)
           fl_set_button(wids.opt[i], RELEASED);
             
       fl_hide_object(wids.current_grp);
       wids.current_grp = wids.pixmap_grp;
       fl_show_object(wids.current_grp);

    }
    
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */ 
/* Callback for choice buttons; key off data that is passed in - UUGHH */
void YesNoCB(FL_OBJECT *ob, long data)
{
    switch (data) {
           case 0: pov.display_during_render = fl_get_choice(ob) - 1; 
                   if (pov.display_during_render == 1) {
                      fl_set_object_color(wids.auto_image, 
                          FL_COL1, FL_INACTIVE_COL); 
                      fl_deactivate_object(wids.auto_image);
                   }
                   else {
                      fl_set_object_color(wids.auto_image, FL_COL1, FL_BLACK); 
                      fl_activate_object(wids.auto_image);
                   }
                   break; 
           case 1: switch (fl_get_choice(ob) - 1) {
                          case 0: pov.verbose = 0;
                                  pov.redirect_on = 0;
                                  break;
                          case 1: pov.verbose = 1;
                                  pov.redirect_on = 0;
                                  break;
                          case 2: pov.verbose = 1;
                                  pov.redirect_on = 1;
                                  break;
                          default: perror("xfpovray: YesNoCB");
                   }
                   break;
           case 2: pov.antialias = fl_get_choice(ob) - 1;
                   break;
           case 3: pov.radiosity = fl_get_choice(ob) - 1;
                   break; 
           case 4: auto_view = fl_get_choice(ob) - 1;
                   break;
           case 5: pov.output_to_file = fl_get_choice(ob) - 1;
                   break;
           case 6: pov.version = fl_get_choice(ob);
                   break;
           case 7: pov.remove_bounds = fl_get_choice(ob) - 1;
                   break;
           case 8: pov.split_unions = fl_get_choice(ob) - 1;
                   break;
           case 9: pov.bounding = fl_get_choice(ob) - 1;
                   break;
           case 10: pov.light_buffer = fl_get_choice(ob) - 1;
                   break;
           case 11: pov.vista_buffer = fl_get_choice(ob) - 1;
                   break;
           case 12: pov.aa_sampling_method = fl_get_choice(ob);
                   break;
           case 13: pov.cyclic_animation = fl_get_choice(ob) - 1;
                   break;
           case 14: switch (fl_get_choice(ob) - 1) {
                          case 0: pov.field_rendering = 0;
                                  break;
                          case 1: pov.field_rendering = 1;
                                  pov.odd_field = 0;
                                  break;
                          case 2: pov.field_rendering = 1;
                                  pov.odd_field = 1;
                                  break;
                          default: perror("xfpovray: YesNoCB");
                   } 
                   break;
           case 15: pov.palette = fl_get_choice(ob);
                   break;
           case 16: pov.output_filetype = fl_get_choice(ob);
                   break;
           case 17: pov.jitter = fl_get_choice(ob) - 1;
                   break;
                      
           default: perror("xfpovray: YesNoCB"); /* Error */

    } /* endswitch */
}
