#!/bin/sh
#
# File: install.sh
#
# Purpose: Sets user configuration for xfpovray, and then 
#          compiles and installs.  From the prompt, 
#          enter ./install.sh
#
# Author: Robert Mallozzi
#         <mallozzir@cspar.uah.edu>
#         1996 July
#
#         RSM: 1996 Aug, added 'make clean?' option
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #


# A temporary output file 
xf_temp_file=xf_temp$$


echo "/* Header file for xfpovray" > $xf_temp_file
echo "** Automatically generated by install.sh" >> $xf_temp_file
echo "**" >> $xf_temp_file
echo "** `date`" >> $xf_temp_file
echo "*/" >> $xf_temp_file
 
install_dir=$PWD

ans="n"
until [ "$ans" = "" -o "$ans" = "y" -o "$ans" = "Y" ]; do

      echo
      echo "Configuration for xfpovray"
      echo
       
      echo "Command used to invoke POV-Ray."
      echo -n " [/usr/local/bin/povray] "
      read pov_command
      if [ "$pov_command" = "" ]; then
         pov_command=/usr/local/bin/povray
      fi     
      echo
      
      echo "System or default library path for POV-Ray include files."
      echo -n " [/usr/local/include/povray] "
      read pov_lib_path
      if [ "$pov_lib_path" = "" ]; then
         pov_lib_path=/usr/local/include/povray
      fi     
      echo

      echo "Personal library path.  Set to NULL if you don't need this."
      echo -n " [NULL] " 
      read pov_personal_lib_path
      if [ "$pov_personal_lib_path" = "" ]; then
         pov_personal_lib_path=NULL
      fi
      echo
   
      echo "Command for your default image viewer."
      echo -n " [xv] "
      read image_viewer
      if [ "$image_viewer" = "" ]; then
         image_viewer=xv
      fi
      echo

      echo "Command for your default editor."
      echo " If you use a console editor like vi, execute the editor"
      echo " in an xterm, e.g., xterm -e vi."
      echo -n " [nedit] "
      read editor
      if [ "$editor" = "" ]; then
         editor=nedit
      fi
      echo

      echo "Path for the startup pixmap (xfpovray.xpm)."  
      echo " This xpm image should be 320x240;"  
      echo " a default image is included in the distribution."
      echo " This must be the permanent path so xfpovray can find the"
      echo " image when the program starts."
      echo -n " [${install_dir}/xfpovray.xpm] "
      read startup_pixmap
      if [ "$startup_pixmap" = "" ]; then
         startup_pixmap=${install_dir}/xfpovray.xpm
      fi
      echo

      echo "Default directory to search for scene files when xfpovray is run."
      echo -n " [.] "
      read pov_dir
      if [ "$pov_dir" = "" ]; then
         pov_dir=.
      fi      
      echo

      echo "Default file filter for scene files."
      echo -n " [*.pov] "
      read scene_filter
      if [ "$scene_filter" = "" ]; then
         scene_filter=\*.pov
      fi      
      echo
      
      echo "Default file filter for image files."
      echo -n " [*.tga] "
      read image_filter
      if [ "$image_filter" = "" ]; then
         image_filter=\*.tga
      fi      
      echo  
  
      echo "Default image width (pixels)."
      echo -n " [150] "
      read image_width
      if [ "$image_width" = "" ]; then
         image_width=150
      fi      
      echo "Default image height (pixels)."
      echo -n " [150] "
      read image_height
      if [ "$image_height" = "" ]; then
         image_height=150
      fi      

      echo
      echo "   POV-Ray command: $pov_command"
      echo "   System library path: $pov_lib_path"
      echo "   Personal library path: $personal_lib_path"
      echo "   Image viewer: $image_viewer"
      echo "   Editor: $editor"
      echo "   Default xpm image: $startup_pixmap"
      echo "   Directory to parse for scene files: $pov_dir"
      echo "   File filter for scene files: $scene_filter"
      echo "   File filter for image files: $image_filter"
      echo "   Default image size: $image_width x $image_height"
      echo "   xfpovray temporary files will be written to"
      echo "     \$HOME/._xfpovray.ini"
      echo "     \$HOME/._xfpovray.stream"
      echo "     \$HOME/._xfpovray.fatal"
      echo
      echo -n "Accept configuration [y]? "

      read ans

      if [ "$ans" = "" -o "$ans" = "y" -o "$ans" = "Y" ]; then
      
         echo " " >> $xf_temp_file

         echo "/* Command to invoke POV-ray */" >> $xf_temp_file
         echo "#define POV_COMMAND \"$pov_command\"" >> $xf_temp_file 
         echo " " >> $xf_temp_file

         echo "/* System or default library path.  You can add up to 4 more on" >> $xf_temp_file
         echo "** the widget for a total of 5, even though POV-Ray supports more." >> $xf_temp_file
         echo "*/" >> $xf_temp_file
         echo "#define POV_LIB_PATH \"$pov_lib_path\"" >> $xf_temp_file         
         echo " " >> $xf_temp_file

         echo "/* Personal library path.  Set to NULL (\"\") if you don't need this */" >> $xf_temp_file 
         echo "#define POV_PERSONAL_LIB_PATH \"$personal_lib_path\"" >> $xf_temp_file
         echo " " >> $xf_temp_file

         echo "/* Temporary files used by xfpovray, written in directory specified" >> $xf_temp_file 
         echo "** by the environment variable HOME" >> $xf_temp_file
         echo "*/" >> $xf_temp_file
         echo "#define _POV_INIFILE \"._xfpovray.ini\"" >> $xf_temp_file 
         echo "#define _STREAM_FILE \"._xfpovray.stream\"" >> $xf_temp_file 
         echo "#define _FATAL_FILE \"._xfpovray.fatal\"" >> $xf_temp_file 
         echo " " >> $xf_temp_file

         echo "/* Image file viewer command */" >> $xf_temp_file
         echo "#define DISPLAY_COMMAND \"$image_viewer\"" >> $xf_temp_file
         echo " " >> $xf_temp_file

         echo "/* Editor command.  If you use a console editor like vi, execute the" >> $xf_temp_file
         echo "** editor in an xterm, e.g., "xterm -e vi"" >> $xf_temp_file
         echo "*/" >> $xf_temp_file
         echo "#define POV_EDITOR \"$editor\"" >> $xf_temp_file
         echo " " >> $xf_temp_file

         echo "/* A startup pixmap.  This xpm image should be 320x240 */" >> $xf_temp_file
         echo "#define STARTUP_PIXMAP \"$startup_pixmap\"" >> $xf_temp_file
         echo " " >> $xf_temp_file

         echo "/* Default directory to search for scene files when xfpovray is run */" >> $xf_temp_file
         echo "#define POV_DIR \"$pov_dir\"" >> $xf_temp_file
         echo " " >> $xf_temp_file
         
         echo "/* Default file filter for scene files */" >> $xf_temp_file
         echo "#define SCENE_FILTER \"$scene_filter\"" >> $xf_temp_file
         echo " " >> $xf_temp_file
         
         echo "/* Default file filter for image files */" >> $xf_temp_file
         echo "#define IMAGE_FILTER \"$image_filter\"" >> $xf_temp_file
         echo " " >> $xf_temp_file
 
         echo "/* Default image size */" >> $xf_temp_file
         echo "#define POV_WIDTH $image_width" >> $xf_temp_file
         echo "#define POV_HEIGHT $image_height" >> $xf_temp_file                
         echo " " >> $xf_temp_file

      else
         
         echo -n "Run configuration again [y]? "
         read result
         if [ "$result" != "" -a "$result" != "y" -a "$result" != "Y" ]; then
            echo
            echo "ERROR: Installation terminated."
            echo
            exit 1
         else
            echo
            echo
         fi           
   
      fi

done

cp $xf_temp_file ${install_dir}/xfpovray.h
rm -f $xf_temp_file

echo -n "OK to execute make [y]? "
read ans

last_errcode=1
if [ "$ans" = "" -o "$ans" = "y" -o "$ans" = "Y" ]; then
   cd $install_dir
   make clean
   make
   last_errcode=$?
fi

if [ $last_errcode -eq 0 ]; then

   echo
   echo -n "OK to make clean [y]? "
   read ans
   if [ "$ans" = "" -o "$ans" = "y" -o "$ans" = "Y" ]; then
      make clean
   fi
   echo
   echo "Compilation complete." 
   echo "Install xfpovray in your binary directory (e.g., /usr/local/bin)."

   exit 0

else
   
   echo
   echo "ERROR: Installation terminated."
   echo
   exit 1
   
fi   
