#include "forms.h"
#include "xfpovray.h"
#include "povray-defaults.h"
#include "wids_struct.h"
#include "pov_struct.h"

extern struct pov_options pov;
extern struct widget_ids wids; 

extern char POV_INIFILE[1024];
extern char STREAM_FILE[1024];
extern char FATAL_FILE[1024];

 
/* Function to write the povray configuration file */
void write_config(void)
{
    char temp[256];    
    FILE *options_file;
    int i;
   
              
    if ((options_file = fopen(POV_INIFILE, "w")) == NULL) {
       perror("xfpovray: write_config");
       exit (1);
    }



    fprintf(options_file, "version=%3.1f\n", (float) pov.version);    

    fprintf(options_file, "input_file_name=%s\n", pov.input_file_name);
    
    /* Library path */
    for (i=0; i<5; i++) {
        if (strcmp(fl_get_input(wids.libpath_input[i]), "") != 0)
           fprintf(options_file, "library_path=%s\n", 
               fl_get_input(wids.libpath_input[i]));
    }
               
    /* Image height and width. 
    ** Must parse the input boxes in case the user 
    ** made an entry by clicking in the input box instead
    ** of using the pulldown menu.
    */
    if (strcmp(fl_get_input(wids.wid_height_input), "") == 0) {
       sprintf(temp, "%d", POV_HEIGHT);
       fl_set_input(wids.wid_height_input, temp);
       pov.height = POV_HEIGHT;
    }
    else
       pov.height = atoi(fl_get_input(wids.wid_height_input));

    if (strcmp(fl_get_input(wids.wid_width_input), "") == 0) {
       sprintf(temp, "%d", POV_WIDTH);
       fl_set_input(wids.wid_width_input, temp);
       pov.width = POV_WIDTH;
    }
    else
       pov.width = atoi(fl_get_input(wids.wid_width_input)); 
       
    fprintf(options_file, "height=%d\n", pov.height); 
    fprintf(options_file, "width=%d\n", pov.width); 

    fprintf(options_file, "verbose=%d\n", pov.verbose);

    /* Redirection options.                   
    ** Turn off redirect to console; redirect to file if selected,
    ** then display file in message browser.
    */
    fprintf(options_file, "all_console=%d\n", 0);
    if (pov.redirect_on == 1)
       fprintf(options_file, "all_file=%s\n", STREAM_FILE); 
    else
       fprintf(options_file, "fatal_file=%s\n", FATAL_FILE);
 
    /* Partial image height and width; parse the input boxes */ 
    if (strcmp(fl_get_input(wids.wid_start_column), "") != 0)
       fprintf(options_file, "start_column=%s\n", 
       fl_get_input(wids.wid_start_column)); 
    if (strcmp(fl_get_input(wids.wid_end_column), "") != 0)
       fprintf(options_file, "end_column=%s\n", 
       fl_get_input(wids.wid_end_column)); 
    if (strcmp(fl_get_input(wids.wid_start_row), "") != 0)
       fprintf(options_file, "start_row=%s\n", 
       fl_get_input(wids.wid_start_row)); 
    if (strcmp(fl_get_input(wids.wid_end_row), "") != 0)
       fprintf(options_file, "end_row=%s\n", 
       fl_get_input(wids.wid_end_row)); 

    /* Mosaic preview; parse the input boxes */
    if (strcmp(fl_get_input(wids.mosaic_start), "") != 0)
       fprintf(options_file, "preview_start_size=%s\n", 
               fl_get_input(wids.mosaic_start)); 
    if (strcmp(fl_get_input(wids.mosaic_end), "") != 0)
       fprintf(options_file, "preview_end_size=%s\n", 
               fl_get_input(wids.mosaic_end)); 
 
  
    fprintf(options_file, "quality=%d\n", pov.quality); 

    fprintf(options_file, "antialias=%d\n", pov.antialias); 
    fprintf(options_file, "antialias_depth=%d\n", pov.aa_depth); 
    fprintf(options_file, "sampling_method=%d\n", pov.aa_sampling_method); 
    fprintf(options_file, "antialias_threshold=%s\n", 
        fl_get_input(wids.antialias_threshold)); 

    fprintf(options_file, "radiosity=%d\n", pov.radiosity); 

    fprintf(options_file, "display=%d\n", pov.display_during_render);
        
    /* If displaying during a render, pause to view image */    
    if (pov.display_during_render == 1)
       fprintf(options_file, "pause_when_done=%d\n", 1); 

    fprintf(options_file, "output_to_file=%d\n", pov.output_to_file);

    fprintf(options_file, "remove_bounds=%d\n", pov.remove_bounds);

    fprintf(options_file, "split_unions=%d\n", pov.split_unions); 

    fprintf(options_file, "bounding=%d\n", pov.bounding);
    
    if (strcmp(fl_get_input(wids.bounding_threshold), "") != 0)
       pov.bounding_threshold = atoi(fl_get_input(wids.bounding_threshold));
    else 
       pov.bounding_threshold = POV_BOUNDING_THRESHOLD; 
    fprintf(options_file, "bounding_threshold=%d\n", pov.bounding_threshold); 

    fprintf(options_file, "light_buffer=%d\n", pov.light_buffer);

    fprintf(options_file, "vista_buffer=%d\n", pov.vista_buffer);

    fprintf(options_file, "clock=%s\n", fl_get_input(wids.clock_input)); 

    fprintf(options_file, "initial_clock=%s\n", 
        fl_get_input(wids.initial_clock_input)); 

    fprintf(options_file, "final_clock=%s\n", 
        fl_get_input(wids.final_clock_input)); 
 
    pov.initial_frame = atoi(fl_get_input(wids.init_frame_input));
    fprintf(options_file, "initial_frame=%d\n", pov.initial_frame);
      
    pov.final_frame = atoi(fl_get_input(wids.final_frame_input));
    fprintf(options_file, "final_frame=%d\n", pov.final_frame);

    if (strcmp(fl_get_input(wids.subset_start_frame_input), "") != 0) {
       fprintf(options_file, "subset_start_frame=%s\n", 
           fl_get_input(wids.subset_start_frame_input));
    }
    if (strcmp(fl_get_input(wids.subset_end_frame_input), "") != 0) {
       fprintf(options_file, "subset_end_frame=%s\n", 
           fl_get_input(wids.subset_end_frame_input));
    }
       
    fprintf(options_file, "cyclic_animation=%d\n", pov.cyclic_animation);

    fprintf(options_file, "field_render=%d\n", pov.field_rendering);
    if (pov.field_rendering == 1)
       fprintf(options_file, "odd_field=%d\n", pov.odd_field);

    switch (pov.palette) {
           case 1: fprintf(options_file, "palette=T\n");
                   break;
           case 2: fprintf(options_file, "palette=H\n");
                   break;
           case 3: fprintf(options_file, "palette=G\n");
                   break;
    }           

    switch (pov.output_filetype) {
           case 1: fprintf(options_file, "output_file_type=T\n");
                   break;
           case 2: fprintf(options_file, "output_file_type=C\n");
                   break;
           case 3: fprintf(options_file, "output_file_type=N\n");
                   break;
           case 4: fprintf(options_file, "output_file_type=P\n");
                   break;
           case 5: fprintf(options_file, "output_file_type=S\n");
                   break;
    }           

    fprintf(options_file, "jitter=%d\n", pov.jitter); 

    fprintf(options_file, "jitter_amount=%s\n", 
        fl_get_input(wids.jitter_input)); 


    fclose(options_file);

 
}
